@@ -60,7 +60,7 @@
 #include "libpq/auth.h"
 #include "libpq/libpq.h"
 
-#define PG_FAILOVER_SLOTS_VERSION "1.0.1"
+#define PG_FAILOVER_SLOTS_VERSION "1.1.0"
 
 PG_MODULE_MAGIC;
 
@@ -70,7 +70,6 @@ PG_MODULE_MAGIC;
 #endif
 
 #define EXTENSION_NAME "pg_failover_slots"
-#define WORKER_NAP_TIME 60000L
 #define WORKER_WAIT_FEEDBACK 10000L
 
 typedef struct RemoteSlot
@@ -104,6 +103,10 @@ List *standby_slot_names = NIL;
 int standby_slots_min_confirmed;
 XLogRecPtr standby_slot_names_oldest_flush_lsn = InvalidXLogRecPtr;
 
+/* Various configuration */
+int worker_nap_time;
+char *pg_failover_maintenance_db;
+
 /* Slots to sync */
 char *pg_failover_slots_dsn;
 char *pg_failover_slot_names;
@@ -443,6 +446,7 @@ get_database_oid(const char *dbname)
 static void
 make_sync_failover_slots_dsn(StringInfo connstr, char *db_name)
 {
+	Assert(db_name);
 	if (pg_failover_slots_dsn && strlen(pg_failover_slots_dsn) > 0)
 	{
 		if (db_name)
@@ -454,8 +458,7 @@ make_sync_failover_slots_dsn(StringInfo connstr, char *db_name)
 	else
 	{
 		Assert(WalRcv);
-		appendStringInfo(connstr, "%s dbname=%s", WalRcv->conninfo,
-						 db_name ? db_name : "postgres");
+		appendStringInfo(connstr, "%s dbname=%s", WalRcv->conninfo, db_name);
 	}
 }
 
@@ -763,7 +766,10 @@ synchronize_one_slot(RemoteSlot *remote_slot)
 		 * We have to create the slot to reserve its name and resources, but
 		 * don't want it to persist if we fail.
 		 */
-#if PG_VERSION_NUM >= 140000
+#if PG_VERSION_NUM >= 170000
+		ReplicationSlotCreate(remote_slot->name, true, RS_EPHEMERAL,
+							  remote_slot->two_phase, false, false);
+#elif PG_VERSION_NUM >= 140000
 		ReplicationSlotCreate(remote_slot->name, true, RS_EPHEMERAL,
 							  remote_slot->two_phase);
 #else
@@ -874,7 +880,6 @@ static long
 synchronize_failover_slots(long sleep_time)
 {
 	List *slots;
-	uint32 nslots = 0;
 	ListCell *lc;
 	PGconn *conn;
 	XLogRecPtr safe_lsn;
@@ -900,7 +905,7 @@ synchronize_failover_slots(long sleep_time)
 	elog(DEBUG1, "starting replication slot synchronization from primary");
 
 	initStringInfo(&connstr);
-	make_sync_failover_slots_dsn(&connstr, NULL /* Use default db name */);
+	make_sync_failover_slots_dsn(&connstr, pg_failover_maintenance_db);
 	conn = remote_connect(connstr.data, "pg_failover_slots");
 
 	/*
@@ -945,6 +950,10 @@ synchronize_failover_slots(long sleep_time)
 			if (!s->in_use || active)
 				continue;
 
+			/* Only check for logical slots. */
+			if (SlotIsPhysical(s))
+				continue;
+
 			/* Try to find slot in slots returned by primary. */
 			foreach (lc, slots)
 			{
@@ -1052,7 +1061,6 @@ synchronize_failover_slots(long sleep_time)
 			remote_slot->restart_lsn = lsn;
 
 		synchronize_one_slot(remote_slot);
-		nslots++;
 	}
 
 	PQfinish(conn);
@@ -1087,14 +1095,14 @@ pg_failover_slots_main(Datum main_arg)
 	while (true)
 	{
 		int rc;
-		long sleep_time = WORKER_NAP_TIME;
+		long sleep_time = worker_nap_time;
 
 		CHECK_FOR_INTERRUPTS();
 
 		if (RecoveryInProgress())
-			sleep_time = synchronize_failover_slots(WORKER_NAP_TIME);
+			sleep_time = synchronize_failover_slots(worker_nap_time);
 		else
-			sleep_time = WORKER_NAP_TIME * 10;
+			sleep_time = worker_nap_time * 10;
 
 		rc =
 			WaitLatch(MyLatch, WL_LATCH_SET | WL_TIMEOUT | WL_POSTMASTER_DEATH,
@@ -1291,8 +1299,8 @@ wait_for_standby_confirmation(XLogRecPtr commit_lsn)
 		}
 
 		/*
-		 * The user might clear bdr.standby_slot_name or change it to a new
-		 * standby. If we don't notice, we'll keep looping indefinitely here,
+		 * The user might modify or clear pg_failover_slots.standby_slot_names.
+		 * If we don't notice, we'll keep looping indefinitely here,
 		 * so we have to check for config changes.
 		 */
 		if (ConfigReloadPending)
@@ -1470,6 +1478,21 @@ _PG_init(void)
 		&pg_failover_slots_dsn, "", PGC_SIGHUP, GUC_SUPERUSER_ONLY, NULL, NULL,
 		NULL);
 
+	DefineCustomIntVariable(
+		"pg_failover_slots.worker_nap_time",
+		"Time to sleep between two synchronisation attempts.",
+		NULL,
+		&worker_nap_time, 60000, 1000, INT_MAX, PGC_SIGHUP,
+		GUC_SUPERUSER_ONLY | GUC_UNIT_MS, NULL, NULL, NULL);
+
+	DefineCustomStringVariable(
+		"pg_failover_slots.maintenance_db",
+		"Database to connect to when using the primary_conninfo",
+		"When connecting to the primary using the primary_conninfo instead of a specifically set "
+		"pg_failover_slots.primary_dsn, use this datbase to query the pg_replication_slots view.",
+		&pg_failover_maintenance_db, "postgres", PGC_SIGHUP, GUC_SUPERUSER_ONLY,
+		NULL, NULL, NULL);
+
 
 	if (IsBinaryUpgrade)
 		return;
